const int pinA = 2; // A channel is connected to pin 2
const int pinB = 3; // B channel is connected to pin 3
const int measureButton = 4; // Measure button is connected to pin 4
const float circumference = 200.0; // Encoder wheel circumference in mm
const int stepsPerRotation = 2000; // Encoder steps per rotation
volatile long count = 0; // Encoder count variable as a long integer
volatile int lastA = 0; // Last state of A channel
bool measurementInProgress = false; // Measurement in progress state

void setup() {
  Serial.begin(9600); // Initialize serial communication
  attachInterrupt(digitalPinToInterrupt(pinA), handleInterrupt, CHANGE); // Attach interrupt for A channel
  pinMode(measureButton, INPUT_PULLUP); // Set up the measure button as an input
}

void loop() {
  if (measurementInProgress) { // Check if measurement is in progress
    float length = (count * circumference) / (stepsPerRotation * 2.0); // Calculate length and divide by 2
    Serial.print("Length: ");
    Serial.print(length);
    Serial.println(" mm");
  }

  // Check if the measure button is pressed
  if (digitalRead(measureButton) == LOW && !measurementInProgress) {
    measurementInProgress = true; // Start measurement
    count = 0; // Reset count to zero
    Serial.println("Measurement started. Release the measure button to stop measuring.");
    delay(500); // Wait for 500ms to debounce the button
  } else if (digitalRead(measureButton) == HIGH && measurementInProgress) {
    measurementInProgress = false; // End measurement
    float length = (count * circumference) / (stepsPerRotation * 2.0); // Calculate length and divide by 2
    Serial.print("Total length: ");
    Serial.print(length);
    Serial.println(" mm");
    count = 0; // Reset count to zero
    delay(500); // Wait for 500ms to debounce the button
  }
}

void handleInterrupt() {
  if (measurementInProgress) { // Only handle interrupt if measurement is in progress
    int currentA = digitalRead(pinA); // Read the current state of A channel
    if (currentA != lastA) { // Check if the state of A channel has changed
      if (digitalRead(pinB) != currentA) { // Check the direction of rotation
        count++;
      } else {
        count--;
      }
      lastA = currentA; // Update the last state of A channel
    }
  }
}
